@@ -2,7 +2,7 @@
 /*! @file apolloron.h
     @brief Header file of libapolloron.
     @author Masashi Astro Tachibana, Apolloron Project.
-    @date 2015-08-15
+    @date 2018-04-11
  ******************************************************************************/
 
 #ifndef _APOLLORON_H_
@@ -14,6 +14,7 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <netinet/in.h>
+#include <string>
 
 namespace apolloron {
 
@@ -23,7 +24,7 @@ class Keys;
 
 /*! @brief Version of libapolloron.
  */
-#define LIBAPOLLORON_VERSION "0.46.3"
+#define LIBAPOLLORON_VERSION "0.46.6"
 
 extern const char *BUILD_DATE;
 extern const char *BUILD_HOST;
@@ -426,6 +427,7 @@ protected:
     long nElementTotal[64];
     Element ***pElement; // (pointers of array) x 64
     String *tmpString; // temporary string
+    String *emptyString; // empty string to retuen if not found
 public:
     Keys();
     Keys(const Keys & keys);
@@ -436,16 +438,18 @@ public:
 
     // Reference of value of element of key
     virtual String& operator [] (const String &key);
+    virtual const String& operator [] (const String &key) const;
     virtual String& operator [] (const char *key);
+    virtual const String& operator [] (const char *key) const;
 
-    virtual String& getValue(const String &key);
-    virtual String& getValue(const char *key);
+    virtual String& getValue(const String &key) const;
+    virtual String& getValue(const char *key) const;
     virtual const char* read(const String &key) const; // for multi-threaded use
     virtual const char* read(const char *key) const; // for multi-threaded use
 
     // Key's existence check
-    virtual bool isKey(const String &key) const;
-    virtual bool isKey(const char *key) const;
+    virtual bool isKeyExist(const String &key) const;
+    virtual bool isKeyExist(const char *key) const;
 
     // Get number of key
     virtual long max() const;
@@ -764,17 +768,63 @@ public:
 };
 
 
+#ifndef _FCGIAPP_H
+typedef struct FCGX_Stream {
+    unsigned char *rdNext;    /* reader: first valid byte
+                               * writer: equals stop */
+    unsigned char *wrNext;    /* writer: first free byte
+                               * reader: equals stop */
+    unsigned char *stop;      /* reader: last valid byte + 1
+                               * writer: last free byte + 1 */
+    unsigned char *stopUnget; /* reader: first byte of current buffer
+                               * fragment, for ungetc
+                               *                                * writer: undefined */
+    int isReader;
+    int isClosed;
+    int wasFCloseCalled;
+    int FCGI_errno;                /* error status */
+    void (*fillBuffProc) (struct FCGX_Stream *stream);
+    void (*emptyBuffProc) (struct FCGX_Stream *stream, int doClose);
+    void *data;
+} FCGX_Stream;
+
+typedef struct FCGX_Request {
+    int requestId;            /* valid if isBeginProcessed */
+    int role;
+    FCGX_Stream *in;
+    FCGX_Stream *out;
+    FCGX_Stream *err;
+    char **envp;
+
+    /* Don't use anything below here */
+
+    struct Params *paramsPtr;
+    int ipcFd;               /* < 0 means no connection */
+    int isBeginProcessed;     /* FCGI_BEGIN_REQUEST seen */
+    int keepConnection;       /* don't close ipcFd at end of request */
+    int appStatus;
+    int nWriters;             /* number of open writers (0..2) */
+    int flags;
+    int listen_sock;
+} FCGX_Request;
+#endif
+
 /*----------------------------------------------------------------------------*/
 /* FCGI class                                                                 */
 /*----------------------------------------------------------------------------*/
 /*! @brief FCGI Class.
  */
-int FCGI_Accept();
-void FCGI_Finish();
+
+int FCGX_Init(void);
+int FCGX_InitRequest(FCGX_Request *request, int sock, int flags);
+int FCGX_Accept_r(FCGX_Request *request);
+void FCGX_Finish_r(FCGX_Request *request);
 
 class FCGI : public CGI {
+protected:
+    FCGX_Request *request;
 public:
-    FCGI();
+    FCGI(FCGX_Request *req);
     virtual ~FCGI();
 
     // Deletion of object instance
@@ -883,8 +933,8 @@ public:
     virtual bool sendFile(const String &local_filename, const String &filename);
     virtual bool appendFile(const String &local_filename, const String &filename);
 
-    virtual Sheet &list(const String &fglob);
-    virtual Sheet &nlist(const String &fglob);
+    virtual Sheet &list();
+    virtual Sheet &nlist();
     virtual bool mkdir(const String &dirname);
     virtual bool mkdirp(const String &dirname);
     virtual bool rmdir(const String &dirname);
