@@ -37,13 +37,13 @@ CGI::CGI() {
             p++;
         }
 
-        if (!strcasecmp(buf, "GET")) {
-            /* GET Method */
+        if (!strcasecmp(buf, "GET") || !strcasecmp(buf, "DELETE")) {
+            /* GET, DELETE Method */
             if ((cpstr = getenv("QUERY_STRING")) != NULL) {
                 str = cpstr;
             }
-        } else if (!strcasecmp(buf, "POST")) {
-            /* POST Method */
+        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT")) {
+            /* POST, PUT Method */
             if ((cpstr = getenv("CONTENT_LENGTH")) != NULL) {
                 int len, getlen, getlen_pre;
                 strncpy(buf, cpstr, 1024);
