@@ -11,16 +11,20 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <ctype.h>
+
+#include "fcgiapp.h"
 #include "apolloron.h"
-#include "fcgi_stdio.h"
 
 namespace apolloron {
 
+#define getenv(name) FCGX_GetParam((name), (*this).request->envp)
+
+
 /*! Constructor of String.
  *  @param void
  *  @return void
  */
-FCGI::FCGI() {
+FCGI::FCGI(FCGX_Request *request) {
     String str;
     char buf[1024 + 1];
     const char *cpstr;
@@ -28,6 +32,9 @@ FCGI::FCGI() {
 
     (*this).clear();
     str.clear();
+
+    (*this).request = request;
+
     if ((cpstr = getenv("REQUEST_METHOD")) != NULL) {
         char *p;
         strncpy(buf, cpstr, 1024);
@@ -38,15 +45,15 @@ FCGI::FCGI() {
             p++;
         }
 
-        if (!strcasecmp(buf, "GET")) {
-            /* GET Method */
+        if (!strcasecmp(buf, "GET") || !strcasecmp(buf, "DELETE")) {
+            /* GET, DELETE Method */
             if ((cpstr = getenv("QUERY_STRING")) != NULL) {
                 str = cpstr;
             }
-        } else if (!strcasecmp(buf, "POST")) {
-            /* POST Method */
+        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT")) {
+            /* POST, PUT Method */
             if ((cpstr = getenv("CONTENT_LENGTH")) != NULL) {
-                int len, getlen, getlen_pre;
+                int len, tmplen, getlen, getlen_pre;
                 strncpy(buf, cpstr, 1024);
                 buf[1024]='\0';
                 len = atoi(buf);
@@ -55,16 +62,20 @@ FCGI::FCGI() {
                 pstr[len] = '\0';
                 while (getlen < len) {
                     getlen_pre = getlen;
-                    getlen += fread(buf, 1, (len - getlen <= 1024)?(len - getlen):1024, stdin);
+                    tmplen = FCGX_GetStr(buf, (len - getlen <= 1024)?(len - getlen):1024, request->in);
+                    if (tmplen < 0) {
+                        break;
+                    }
+                    getlen += tmplen;
                     buf[getlen - getlen_pre] = '\0';
                     if (0 < (getlen - getlen_pre)) {
                         memcpy(pstr + getlen_pre, buf, getlen - getlen_pre);
                     } else if (getlen == getlen_pre) {
                         sleep(1);
                     }
-                    if (ferror(stdin)) {
+                    if (request->in->isClosed) {
                         break;
-                    } else if (feof(stdin)) {
+                    } else if (FCGX_HasSeenEOF(request->in)) {
                         break;
                     }
                 }
@@ -487,6 +498,7 @@ bool FCGI::clearHeader() {
  */
 bool FCGI::putHeader() {
     long i, max;
+    const char *cpstr;
 
     if ((*this).putHead == true) {
         return true;
@@ -495,11 +507,12 @@ bool FCGI::putHeader() {
     max = header.max();
 
     for (i = 0; i < max; i++) {
-        fputs(header[i].c_str(), stdout);
-        fputs("\r\n", stdout);
+        cpstr = header[i].c_str();
+        FCGX_PutStr(cpstr, strlen(cpstr), (*this).request->out);
+        FCGX_PutStr("\r\n", 2, (*this).request->out);
     }
 
-    fputs("\r\n", stdout);
+    FCGX_PutStr("\r\n", 2, (*this).request->out);
     (*this).putHead = true;
 
     return true;
@@ -537,7 +550,7 @@ bool FCGI::putContent(const char *content, long size) {
 
     (*this).putHeader();
 
-    fwrite(content, (size_t)size, 1, stdout);
+    FCGX_PutStr(content, (int)size, (*this).request->out);
 
     return true;
 }
@@ -573,6 +586,7 @@ bool FCGI::putError(const char *err_str, long size) {
     }
 
     fwrite(err_str, (size_t)size, 1, stderr);
+    FCGX_PutStr(err_str, (int)size, (*this).request->err);
 
     return true;
 }
