@@ -55,6 +55,11 @@ FTPStream::~FTPStream() {
 }
 
 
+/*! Set timeout
+   @param sec   timeout seconds
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::setTimeout(int sec) {
     if (sec < 0) return false;
     (*this).timeout = sec;
@@ -62,6 +67,14 @@ bool FTPStream::setTimeout(int sec) {
 }
 
 
+/*! Login
+   @param uesr    user
+   @param passwd  password
+   @param host    host
+   @param port    port
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::login(const String &user, const String &passwd,
                       const String &host, const String &port) {
     int ret;
@@ -91,6 +104,11 @@ bool FTPStream::login(const String &user, const String &passwd,
 }
 
 
+/*! Logout
+   @param void
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::logout() {
     if ((*this).loggedIn == true) {
         ftp_quit(&((*this).ftp_obj));
@@ -100,11 +118,21 @@ bool FTPStream::logout() {
 }
 
 
+/*! Check login status
+   @param void
+   @retval true  user logged in
+   @retval false user logged out
+ */
 bool FTPStream::isLoggedIn() {
     return (*this).loggedIn;
 }
 
 
+/*! Change to pasive mode or non passive mode
+   @param pasv_mode  passive: true, non-passive: false
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::passive(bool pasv_mode) {
     if ((*this).loggedIn == false) return false;
     ftp_passive(&((*this).ftp_obj), ((*this).pasvMode == true)?1:0);
@@ -113,11 +141,21 @@ bool FTPStream::passive(bool pasv_mode) {
 }
 
 
+/*! check pasive mode or non passive mode
+   @param void
+   @retval true  passive mode
+   @retval false non-passive mode
+ */
 bool FTPStream::isPassive() {
     return (*this).pasvMode;
 }
 
 
+/*! Reset
+   @param void
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::reset() {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -126,6 +164,11 @@ bool FTPStream::reset() {
 }
 
 
+/*! Noop
+   @param void
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::noop() {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -134,6 +177,11 @@ bool FTPStream::noop() {
 }
 
 
+/*! Site
+   @param cmdline  command line
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::site(const String &cmdline) {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -142,6 +190,11 @@ bool FTPStream::site(const String &cmdline) {
 }
 
 
+/*! Delete
+   @param filename  filename to delete
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::dele(const String &filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -150,6 +203,12 @@ bool FTPStream::dele(const String &filename) {
 }
 
 
+/*! Rename
+   @param from  original filename
+   @param to    filename to rename
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::rename(const String &from, const String &to) {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -158,6 +217,10 @@ bool FTPStream::rename(const String &from, const String &to) {
 }
 
 
+/*! Receive
+   @param filename  filename to get data
+   @return received data
+ */
 String &FTPStream::receiveBuffer(const String &filename) {
     int ret;
     char *dup_buf;
@@ -168,51 +231,89 @@ String &FTPStream::receiveBuffer(const String &filename) {
         (*this).tmpString = dup_buf;
         free(dup_buf);
     }
+    if (ret != FTP_NOERROR) {
+        (*this).tmpString = "";
+    }
     return (*this).tmpString;
 }
 
 
+/*! Send file
+   @param buf       data to send
+   @param filename  filename to send
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::sendBuffer(const String &buf, const String &filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
     ret = ftp_put_buffer(&((*this).ftp_obj), buf, buf.len(), filename.c_str());
-    return true;
+    return (ret == FTP_NOERROR)?true:false;
 }
 
 
+/*! Append data to file
+   @param buf       data to append
+   @param filename  filename to append
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::appendBuffer(const String &buf, const String &filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
     ret = ftp_append_buffer(&((*this).ftp_obj), buf, buf.len(), filename.c_str());
-    return true;
+    return (ret == FTP_NOERROR)?true:false;
 }
 
 
+/*! Receive as a local file
+   @param filename       filename to receive
+   @param local_filename local filename to save
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::receiveFile(const String &filename, const String &local_filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
     ret = ftp_get_file(&((*this).ftp_obj), filename.c_str(), local_filename.c_str());
-    return true;
+    return (ret == FTP_NOERROR)?true:false;
 }
 
 
+/*! Send local file
+   @param local_filename local filename to send
+   @param filename       filename to send
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::sendFile(const String &local_filename, const String &filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
     ret = ftp_put_file(&((*this).ftp_obj), local_filename.c_str(), filename.c_str());
-    return true;
+    return (ret == FTP_NOERROR)?true:false;
 }
 
 
+/*! Append local file
+   @param local_filename local filename to append
+   @param filename       filename to append
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::appendFile(const String &local_filename, const String &filename) {
     int ret;
     if ((*this).loggedIn == false) return false;
     ret = ftp_append_file(&((*this).ftp_obj), local_filename.c_str(), filename.c_str());
-    return true;
+    return (ret == FTP_NOERROR)?true:false;
 }
 
 
-Sheet &FTPStream::list(const String &fglob) {
+/*! Get file list
+   @param void
+   @retval true  success
+   @retval false failure
+ */
+Sheet &FTPStream::list() {
     int ret;
     char *list_data, *saveptr, *line;
     int line_len, row, i;
@@ -342,7 +443,12 @@ Sheet &FTPStream::list(const String &fglob) {
 }
 
 
-Sheet &FTPStream::nlist(const String &fglob) {
+/*! Get file list by nlst
+   @param void
+   @retval true  success
+   @retval false failure
+ */
+Sheet &FTPStream::nlist() {
     int ret;
     char *list_data, *saveptr, *line;
     int line_len, row, i;
@@ -360,12 +466,6 @@ Sheet &FTPStream::nlist(const String &fglob) {
             if ((*this).ftp_obj.reply_code == 450) {
                 /* no files found on this directory */
                 (*this).tmpSheet.addCol("FILENAME");
-                (*this).tmpSheet.addCol("SIZE");
-                (*this).tmpSheet.addCol("DATE");
-                (*this).tmpSheet.addCol("OWNER_UID");
-                (*this).tmpSheet.addCol("OWNER_GID");
-                (*this).tmpSheet.addCol("PERMISSION");
-                (*this).tmpSheet.addCol("IS_DIRECTORY");
                 return (*this).tmpSheet;
             }
         }
@@ -373,12 +473,6 @@ Sheet &FTPStream::nlist(const String &fglob) {
     }
 
     (*this).tmpSheet.addCol("FILENAME");
-    (*this).tmpSheet.addCol("SIZE");
-    (*this).tmpSheet.addCol("DATE");
-    (*this).tmpSheet.addCol("OWNER_UID");
-    (*this).tmpSheet.addCol("OWNER_GID");
-    (*this).tmpSheet.addCol("PERMISSION");
-    (*this).tmpSheet.addCol("IS_DIRECTORY");
 
     /* skip "total xx" line */
     if (!strncmp(list_data, "total ", 6)) {
@@ -472,6 +566,11 @@ Sheet &FTPStream::nlist(const String &fglob) {
 }
 
 
+/*! Create a directory
+   @param dirname  directory name to create
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::mkdir(const String &dirname) {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -480,6 +579,11 @@ bool FTPStream::mkdir(const String &dirname) {
 }
 
 
+/*! Create a directory (recursive)
+   @param dirname  directory name to create
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::mkdirp(const String &dirname) {
     char *pathlist[1000], *p;
     int i, j, len, err_status, ret;
@@ -527,6 +631,11 @@ bool FTPStream::mkdirp(const String &dirname) {
 }
 
 
+/*! Remove a directory
+   @param dirname  directory name to remove
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::rmdir(const String &dirname) {
     int ret;
     if ((*this).loggedIn == false) return false;
@@ -534,6 +643,12 @@ bool FTPStream::rmdir(const String &dirname) {
     return (ret == FTP_NOERROR)?true:false;
 }
 
+
+/*! Get current directory path
+   @param void
+   @retval true  success
+   @retval false failure
+ */
 String &FTPStream::pwd() {
     int ret;
     char buf[4096];
@@ -541,11 +656,20 @@ String &FTPStream::pwd() {
     if ((*this).loggedIn == false) return (*this).tmpString;
     ret = ftp_pwd(&((*this).ftp_obj), buf, 4095);
     buf[4095] = '\0';
-    (*this).tmpString = buf;
+    if (ret != FTP_NOERROR) {
+        (*this).tmpString = "";
+    } else {
+        (*this).tmpString = buf;
+    }
     return (*this).tmpString;
 }
 
 
+/*! Change current directory
+   @param dirname  directory name
+   @retval true  success
+   @retval false failure
+ */
 bool FTPStream::cd(const String &dirname) {
     int ret;
     if ((*this).loggedIn == false) return false;
