@@ -135,7 +135,7 @@ regex_region_copy(RegRegion* r1, RegRegion* r2)
 
 /** stack **/
 #define INVALID_STACK_INDEX   -1
-typedef int  StackIndex;
+typedef long  StackIndex;
 
 typedef struct _StackType {
     int type;
